;batconfg.lsp
;Configures SLaTeX batfile/shellscript (CL version)
;(c) Dorai Sitaram, Rice U., 1991, 1994

#+gcl
(or (find-package :slatex) (make-package :slatex))

#-gcl
(defpackage slatex (:use cl))

(set-dispatch-macro-character #\# #\t
  #'(lambda (p ig ig2)
      (declare (ignore ig ig2))
      t))

(set-dispatch-macro-character #\# #\f
  #'(lambda (p ig ig2)
      (declare (ignore ig ig2))
      nil))

(format t "~&Beginning configuring command script -- wait...")

(defvar *slatex-directory* (directory-namestring *load-pathname*))

(defvar *op-sys*)
(defvar cl-pathname)
(defvar slatex-pathname)
(defvar texinputs)
(defvar texinputs-list)
(defvar accepts-echo)
(defvar accepts-cmdline-file)
(defvar accepts-initfile)
(defvar system-procedure nil)

#+clisp
(setf system-procedure 'run-shell-command)

(with-open-file (inp (concatenate 'string *slatex-directory*
                                  "config.dat")
                     :direction :input)
  (read inp) ;we already know dialect
  (setf *op-sys* (read inp)
	cl-pathname (read inp)
	slatex-pathname (read inp)
	texinputs (read inp)
	texinputs-list (read inp)
	accepts-echo (read inp)
	accepts-cmdline-file (read inp)
	accepts-initfile (read inp))  )

(defvar bat-file)
(setf bat-file
  (concatenate 'string *slatex-directory*
	       (case *op-sys*
		 ((os2 os2fat) "slatex.cmd")
		 (dos "slatex.bat")
		 (unix "slatex"))))

(unless (eq *op-sys* 'mac-os)
  (if (probe-file bat-file) (delete-file bat-file)))

(with-open-file (op bat-file :direction :output)
  (case *op-sys*
    ((unix)
     (cond (accepts-echo
            (format op "~
echo '(load ~s)
(setq slatex::*texinputs* ~s)
(setq slatex::*texinputs-list* `~s)
(slatex::process-main-tex-file \"'$1'\")' | ~a~%"
                    slatex-pathname
                    texinputs
                    texinputs-list
                    cl-pathname))
           ((or accepts-cmdline-file accepts-initfile)
            (let ((f (if accepts-cmdline-file "slatex.jnk" accepts-initfile)))
              (format op "~
echo '(load ~s)' > ~a
echo '(setq slatex::*texinputs* ~s)' >> ~a
echo '(setq slatex::*texinputs-list* `~s)' >> ~a
echo '(slatex::process-main-tex-file \"'$1'\")' >> ~a
echo '(slatex::exit-scheme)' >> ~a
~a ~a
rm ~a~%"
                      slatex-pathname f
                      texinputs f
                      texinputs-list f
                      f
                      f
                      cl-pathname
                      (if accepts-cmdline-file
			  (concatenate 'string " " accepts-cmdline-file " " f)
			  "")
                      f)))
           (else
            (format t "Your CL's system interface isn't very cooperative --
use callsla.scm.~%")))
     (format op "~
if test -f pltexchk.jnk
then tex $1; rm pltexchk.jnk
else latex $1
fi~%"))
    ((dos os2fat os2)
     (format op "@echo off~%")
     (cond ((or accepts-echo accepts-cmdline-file accepts-initfile)
            (let ((f (cond ((or accepts-echo accepts-cmdline-file)
                            "slatex.jnk")
                           (t accepts-initfile))))
              (format op "~
echo (load ~s) > ~a
echo (setq slatex::*texinputs* ~s) >> ~a
echo (setq slatex::*texinputs-list* `~s) >> ~a
echo (slatex::process-main-tex-file \"%1\") >> ~a
echo (slatex::exit-scheme) >> ~a~%"
                      slatex-pathname f
                      texinputs f
                      texinputs-list f
                      f f)
              (cond (accepts-echo
                     (format op "~
echo (load ~s) | ~a~%" f cl-pathname))
                    (accepts-cmdline-file
                     (format op "~
call ~a ~a ~a~%" cl-pathname accepts-cmdline-file f))
                    (accepts-initfile
                     (format op "~
call ~a~%" cl-pathname)))
              (format op "del ~a
if exist pltexchk.jnk goto one
goto two
:one
call tex %1
del pltexchk.jnk
goto end
:two
call latex %1
:end~%" f)))
           (else
            (format t "Your CL's system interface isn't very cooperative --
use callsla.scm.~%"))))))

(defvar callsla-file
  (concatenate 'string *slatex-directory* "callsla.scm"))

(if (probe-file callsla-file)
  (delete-file callsla-file))

(with-open-file (op callsla-file :direction :output)
  (pprint
   `(defpackage slatex (:use cl)) op)
  (pprint
   `(let (slatex-loaded-p)
      (defun call-slatex (texf)
	(let ((texf (namestring texf)))
	  (unless slatex-loaded-p
            (format t "~&Loading SLaTeX...")
	    (load ,slatex-pathname)
            (format t "done.~%")
	    (setf slatex::*texinputs* ,texinputs
		  slatex-loaded-p t))
	  (slatex::process-main-tex-file texf)
	  ,(if system-procedure
	       `(,system-procedure
		 (concatenate 'string
		   (if slatex::*latex?* "latex " "tex ") texf))
	     `(format t "Call ~aTeX on ~a~%"
		      (if slatex::*latex?* "La" "") texf))
	  t)))
   op))

(format t "~&Finished configuring command script.~%")

(when (eq *op-sys* 'unix)
  #+clisp
  (run-shell-command "chmod +x slatex")
  #+gcl
  (system "chmod +x slatex")
  #-(or gcl clisp)
  (format t "~&Type (chmod +x slatex) on Unix command line~%"))
