;batconfg.scm;-*-lisp-*-
;Configures SLaTeX batfile/shellscript (Scheme version)
;(c) Dorai Sitaram, Rice U., 1991, 1994

(load "format.scm")

(pm-format #t "Beginning configuring command script -- wait...~%")

(define dialect 'forward)
(define *op-sys* 'forward)
(define scheme-pathname 'forward)
(define slatex-pathname 'forward)
(define texinputs 'forward)
(define texinputs-list 'forward)
(define accepts-echo 'forward)
(define accepts-cmdline-file 'forward)
(define accepts-initfile 'forward)
(define system-procedure #f)

(call-with-input-file "config.dat"
  (lambda (ip)
    (set! dialect (read ip))
    (set! *op-sys* (read ip))
    (set! scheme-pathname (read ip))
    (set! slatex-pathname (read ip))
    (set! texinputs (read ip))
    (set! texinputs-list (read ip))
    (set! accepts-echo (read ip))
    (set! accepts-cmdline-file (read ip))
    (set! accepts-initfile (read ip))
    (cond ((or (eof-object? dialect)
             (eof-object? *op-sys*)
             (eof-object? scheme-pathname)
             (eof-object? slatex-pathname)
             (eof-object? texinputs)
             (eof-object? texinputs-list)
             (eof-object? accepts-echo)
             (eof-object? accepts-cmdline-file)
             (eof-object? accepts-initfile))
            (error "config.dat has too few answers"))
      ((eof-object? (read ip)) #t)
      (else (error "config.dat has too many answers")))))

(case dialect
  ((chez guile mzscheme scm) (set! system-procedure 'system)))

(define pp
  ;pretty-printer -- not really needed, so use write for dialects
  ;that don't have it
  (case dialect
    ((chez) pretty-print)
    ((guile scm) (if (defined? pretty-print) pretty-print write))
    (else write)))

(define bat-file 'forward)

(case *op-sys*
  ((os2 os2bat)
   (set! bat-file "slatex.cmd"))
  ((dos)
   (set! bat-file "slatex.bat"))
  ((unix)
   (set! bat-file "slatex")))

(if (eq? dialect 'guile)
  (if (not (defined? delete-file))
    (define delete-file %delete-file)))

(if (memq *op-sys* '(unix dos os2fat os2))
    (if (memq dialect '(bigloo chez cscheme guile mzscheme pcsge scm))
        (if (file-exists? bat-file)
            (delete-file bat-file))))

(call-with-output-file bat-file
  (lambda (op)
    (case *op-sys*
      ((unix)
        (if (eq? dialect 'bigloo)
          (begin
            (display "SLaTeX \"$@\" " op)
            (newline op))
          (begin
            (cond (accepts-echo
                    (pm-format op "~
echo '(load ~s)
(set! slatex::*texinputs* ~s)
(set! slatex::*texinputs-list* `~s)
(slatex::process-main-tex-file \"'$1'\")' | ~a~%"
                      slatex-pathname
                      texinputs
                      texinputs-list
                      scheme-pathname))
              ((or accepts-cmdline-file accepts-initfile)
                (let ((f (if accepts-cmdline-file "slatex.jnk"
                           accepts-initfile)))
                  (pm-format op "~
echo '(load ~s)' > ~a
echo '(set! slatex::*texinputs* ~s) >> ~a
echo '(set! slatex::*texinputs-list* `~s) >> ~a
echo '(slatex::process-main-tex-file \"'$1'\") >> ~a
echo '(slatex::exit-scheme)' >> ~a
~a ~a
rm ~a~%"
                    slatex-pathname f
                    texinputs f
                    texinputs-list f
                    f f
                    scheme-pathname
                    (if accepts-cmdline-file
                      (string-append " " accepts-cmdline-file " " f)
                      "")
                    f)))
              (else
                (pm-format #t "Your Scheme's system interface isn't very cooperative --
use callsla.scm.~%")))
            (pm-format op "~
if test -f pltexchk.jnk
then tex $1; rm pltexchk.jnk
else latex $1
fi~%"))))
      ((dos os2fat os2)
        (pm-format op "@echo off~%")
        (cond ((or accepts-echo accepts-cmdline-file accepts-initfile)
                (let ((f (cond ((or accepts-echo accepts-cmdline-file)
                                 "slatex.jnk")
                           (else accepts-initfile))))
                  (pm-format op "~
echo (load ~s) > ~a
echo (set! slatex::*texinputs* ~s) >> ~a
echo (set! slatex::*texinputs-list* `~s) >> ~a
echo (slatex::process-main-tex-file \"%1\") >> ~a
echo (slatex::exit-scheme) >> ~a~%"
                    slatex-pathname f
                    texinputs f
                    texinputs-list f
                    f f)
                  (cond (accepts-echo
                          (pm-format op "~
echo (load ~s) | ~a~%" f scheme-pathname))
                    (accepts-cmdline-file
                      (pm-format op "~
call ~a ~a ~a~%" scheme-pathname accepts-cmdline-file f))
                    (accepts-initfile
                      (pm-format op "~
call ~a~%" scheme-pathname)))
                  (pm-format op "del ~a
if exist pltexchk.jnk goto one
goto two
:one
call tex %1
del pltexchk.jnk
goto end
:two
call latex %1
:end~%" f)))
          (else
            (pm-format #t "Your Scheme's system interface isn't very cooperative --
use callsla.scm.~%")))))))

(if (memq dialect '(chez cscheme guile mzscheme pcsge scm))
    (if (file-exists? "callsla.scm")
	(delete-file "callsla.scm")))

(call-with-output-file "callsla.scm"
  (lambda (op)
    (pp
     `(define call-slatex
	(let ((slatex-not-loaded? #t))
	  (lambda (texf)
	    (if slatex-not-loaded?
		(begin
		  (display "Loading SLaTeX...")
		  (load ,slatex-pathname)
		  (display 'done)
		  (newline)
		  (set! slatex::*texinputs* ,texinputs)
		  (set! slatex-not-loaded? #f)))
	    (slatex::process-main-tex-file texf)
	    ,(if system-procedure
		 `(,system-procedure
		   (string-append
		    (if slatex::*latex?* "latex " "tex ")
		    texf))
		 `(begin
		    (display "Call (la)tex on ")
		    (display texf)
		    (newline)))
	    #t)))
     op)))

(newline)
(display "Finished configuring batfile/shellscript")
(newline)
(if (eq? *op-sys* 'unix)
    (case system-procedure
      ((system) (system "chmod +x slatex"))
      (else
       (display "Type (chmod +x slatex) on Unix command line")
       (newline))))
