;format.scm
;Dorai Sitaram, March 1996

;This is a very small subset of Common Lisp's format.
;Directives processed:

;~s S-expression;
;~a Aesthetic (formerly Ascii);
;~b Binary;
;~o Octal;
;~d Decimal;
;~x heXadecimal;
;~~ tilde itself;
;~% newline;
;~& newline (CL would use freshline here -- we'll include this
;   directive anyway so the same code will work in both
;   languages, correctly in CL and slightly incorrectly but
;   non-error-causingly in Scheme);
;~| pagebreak;
;~t tab;
;~newline eat all ensuing whitespace in the format string
;         (actually CLtL2 says to eat only _non_-newline
;	  whitespace, but it doesn't seem worth the effort).

(define pm-format
  (lambda (p s . aa)
    (let ((format2
	   (lambda (p)
	     (let ((n (string-length s)))
	       (let loop ((i 0) (aa aa))
		 (if (< i n)
		     (let ((c (string-ref s i)))
		       (if (char=? c #\~)
			   (let ((c (string-ref s (+ i 1))))
			     (case c
			       ((#\s #\a #\b #\o #\d #\x)
				(case c
				  ((#\s #\a) ((case c
						((#\s) write)
						((#\a) display))
					      (car aa) p))
				  ((#\b #\o #\d #\x)
				   (display
				    (number->string
				     (car aa) (case c
						((#\b) 2)
						((#\o) 8)
						((#\d) 10)
						((#\x) 16))) p)))
				(loop (+ i 2) (cdr aa)))
			       ((#\~ #\% #\& #\| #\t)
				(display (case c
					   ((#\~) c)
					   ((#\% #\&) #\newline)
					   ((#\|) #\page)
					   ((#\t) #\tab)) p)
				(loop (+ i 2) aa))
			       ((#\newline)
				(let loop2 ((j (+ i 2)))
				  (if (< j n)
				      ;Actually, CLtL2 says to
				      ;ignore only _non_-newline
				      ;whitespace...
				      (if (char-whitespace? (string-ref s j))
					  (loop2 (+ j 1))
					  (loop j aa)))))
			       (else (error "pm-format: Unknown tilde ~a" c))))
			   (begin
			     (display c p)
			     (loop (+ i 1) aa))))))))))
      (case p
	((#t) (format2 (current-output-port)))
	((#f) (call-with-output-string format2))
	(else (format2 p))))))
